//-----------------------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//-----------------------------------------------------------------------------
using System;

using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;

using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;

namespace SamlTools
{
    /// <summary>
    /// Custom SecurityTokenManager that creates a custom SecurityTokenProvider
    /// </summary>
    public class SamlSecurityTokenManager : ClientCredentialsSecurityTokenManager
    {
        /// <summary>
        /// The SamlClientCredentials that created us
        /// </summary>
        SamlClientCredentials samlClientCredentials;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="samlClientCredentials">The creating SamlClientCredentials instance</param>
        public SamlSecurityTokenManager(SamlClientCredentials samlClientCredentials)
            : base(samlClientCredentials)
        {
            // Store the creating client credentials
            this.samlClientCredentials = samlClientCredentials;
        }

        /// <summary>
        /// Creates the custom SecurityTokenProvider when SAML tokens are specified in the tokenRequirement
        /// </summary>
        /// <param name="tokenRequirement">A SecurityTokenRequirement  </param>
        /// <returns>The appropriate SecurityTokenProvider</returns>
        public override SecurityTokenProvider CreateSecurityTokenProvider(SecurityTokenRequirement tokenRequirement)
        {
            if (tokenRequirement.TokenType == SecurityTokenTypes.Saml ||
                tokenRequirement.TokenType == "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV1.1")
            {
                SamlAssertion assertion = this.samlClientCredentials.Assertion;
                SecurityToken prooftoken = this.samlClientCredentials.ProofToken;

                if (assertion == null || prooftoken == null)
                {
                    SecurityBindingElement sbe = null;
                    SecurityAlgorithmSuite sas = null;

                    if (tokenRequirement.TryGetProperty<SecurityBindingElement>("http://schemas.microsoft.com/ws/2006/05/servicemodel/securitytokenrequirement/SecurityBindingElement", out sbe))
                    {
                        sas = sbe.DefaultAlgorithmSuite;
                    }

                    assertion = SamlUtilities.CreateAsymmetricKeyBasedAssertion(sas);

                }

                return new SamlSecurityTokenProvider(assertion, prooftoken);
            }
            else
            {
                return base.CreateSecurityTokenProvider(tokenRequirement);
            }                
        }
    }
}
