//-----------------------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//-----------------------------------------------------------------------------
using System;

using System.Collections.Generic;

using System.IdentityModel.Claims;
using System.IdentityModel.Tokens;

using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace SamlTools
{
	public class SamlUtilities
	{


        public static SamlAssertion CreateAsymmetricKeyBasedAssertion(SecurityAlgorithmSuite algoSuite)
        {

            SecurityToken signatureToken = CreateAsymmetricSignatureToken();

            SecurityKey signatureKey = signatureToken.SecurityKeys[0];
            SecurityKeyIdentifier signatureKeyIdentifier = new SecurityKeyIdentifier(
                signatureToken.CreateKeyIdentifierClause<X509RawDataKeyIdentifierClause>());

            return CreateAssertion20(signatureKey, signatureKeyIdentifier, algoSuite);
        }

        /// <summary>
        /// The following method creates a SAML assertion representing the identity of the user to be represented in AMXBPM - by default this is John Eustace
        /// </summary>
        /// <param name="signatureKey"></param>
        /// <param name="signatureKeyIdentifier"></param>
        /// <param name="algoSuite"></param>
        /// <returns></returns>
        private static SamlAssertion CreateAssertion20(SecurityKey signatureKey, SecurityKeyIdentifier signatureKeyIdentifier, SecurityAlgorithmSuite algoSuite)
        {
            List<string> confirmationMethods = new List<string>(1);

            confirmationMethods.Add("urn:oasis:names:tc:SAML:2.0:cm:sender-vouches");

            SamlSubject samlSubject = null;
            samlSubject = new SamlSubject(
               "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress",
               "www.tibco.com",
               "John Eustace",
               confirmationMethods,
               null,
               null);

            SamlAuthenticationStatement samlAuthStatement = new SamlAuthenticationStatement(
                samlSubject,
                "urn:oasis:names:tc:SAML:2.0:ac:classes:Password",
                DateTime.UtcNow,
                null,
                null,
                null);

            List<SamlStatement> samlSubjectStatements = new List<SamlStatement>();
            samlSubjectStatements.Add(samlAuthStatement);

            SigningCredentials signingCredentials = new SigningCredentials(signatureKey,
                                                                           algoSuite.DefaultAsymmetricSignatureAlgorithm,
                                                                           algoSuite.DefaultDigestAlgorithm,
                                                                           signatureKeyIdentifier);

            DateTime issueInstant = DateTime.UtcNow;

            SamlConditions conditions = new SamlConditions(issueInstant - TimeSpan.FromDays(20), issueInstant + TimeSpan.FromDays(20));

            SamlAssertion samlAssertion = new SamlAssertion("_" + Guid.NewGuid().ToString(),
                                                            "urn:cit.per.tibco.com",
                                                            issueInstant,
                                                            conditions,
                                                            null,
                                                            samlSubjectStatements
                                                            );

            samlAssertion.SigningCredentials = signingCredentials;

            return samlAssertion;
        }

        public static SecurityToken CreateAsymmetricSignatureToken()
        {
            SecurityToken token = null;
            X509Store certStore = new X509Store(StoreName.TrustedPeople, StoreLocation.CurrentUser);
            certStore.Open(OpenFlags.ReadOnly);
            try
            {
                X509Certificate2Collection certs = certStore.Certificates.Find(X509FindType.FindBySubjectName, "SAML", false);
                if (certs != null)
                    token = new X509SecurityToken(certs[0]);
            }
            finally
            {
                certStore.Close();
            }

            return token;
        }    

        private SamlUtilities(){}
	}
}
